@@ -29,6 +29,7 @@ import           Data.Text                          (Text)
 import qualified Data.Text                          as T
 import qualified Data.Text.Encoding                 as T
 import qualified Data.Vector                        as V
+import           Text.Printf
 import qualified Text.XmlHtml                       as X
 import qualified Text.XmlHtml.HTML.Meta             as X
 ------------------------------------------------------------------------------
@@ -132,7 +133,8 @@ runDocumentFile tpath df = do
                    X.XmlDocument _ _ _ -> Xml
                    X.HtmlDocument _ _ _ -> Html
     modifyHS (\hs -> hs { _curMarkup = markup })
-    addDoctype $ maybeToList $ X.docType $ dfDoc df
+    let inDoctype = X.docType $ dfDoc df
+    addDoctype $ maybeToList inDoctype
     modifyHS (setCurTemplateFile curPath .  setCurContext tpath)
     res <- runNodeList nodes
     dt <- getsHS (listToMaybe . _doctypes)
@@ -161,9 +163,25 @@ compileTemplates
     -> IO (Either [String] (HeistState n))
 compileTemplates hs = do
     (tmap, hs') <- runHeistT compileTemplates' (X.TextNode "") hs
+    let pre = _splicePrefix hs'
+    let nsErr = if not (T.null pre) && (_numNamespacedTags hs' == 0)
+                  then Left [noNamespaceSplicesMsg $ T.unpack pre]
+                  else Right ()
     return $ case _spliceErrors hs' of
-               [] -> Right $! hs { _compiledTemplateMap = tmap }
-               es -> Left $ map T.unpack es
+               [] -> nsErr >> (Right $! hs { _compiledTemplateMap = tmap })
+               es -> Left $ either (++) (const id) nsErr $ map T.unpack es
+
+
+------------------------------------------------------------------------------
+noNamespaceSplicesMsg :: String -> String
+noNamespaceSplicesMsg pre = unwords
+    [ printf "You are using a namespace of '%s', but you don't have any" ns
+    , printf "tags starting with '%s'.  If you have not defined any" pre
+    , "splices, then change your namespace to the empty string to get rid"
+    , "of this message."
+    ]
+  where
+    ns = reverse $ drop 1 $ reverse pre
 
 
 ------------------------------------------------------------------------------
@@ -178,6 +196,7 @@ compileTemplates' = do
     foldM runOne H.empty tpathDocfiles
   where
     runOne tmap (tpath, df) = do
+        modifyHS (\hs -> hs { _doctypes = []})
         !mHtml <- compileTemplate tpath df
         return $! H.insert tpath (mHtml, mimeType $! dfDoc df) tmap
 
@@ -253,6 +272,9 @@ lookupSplice nm = do
 -- compileNode to generate the appropriate runtime computation.
 runNode :: Monad n => X.Node -> Splice n
 runNode node = localParamNode (const node) $ do
+    pre <- getsHS _splicePrefix
+    let hasPrefix = (T.isPrefixOf pre `fmap` X.tagName node) == Just True
+    when (not (T.null pre) && hasPrefix) incNamespacedTags
     isStatic <- subtreeIsStatic node
     markup <- getsHS _curMarkup
     if isStatic
@@ -526,8 +548,9 @@ bindSplice :: Text             -- ^ tag name
            -> HeistState n     -- ^ source state
            -> HeistState n
 bindSplice n v ts =
-    ts { _compiledSpliceMap = H.insert n v (_compiledSpliceMap ts) }
-
+    ts { _compiledSpliceMap = H.insert n' v (_compiledSpliceMap ts) }
+  where
+    n' = _splicePrefix ts `mappend` n
 
 ------------------------------------------------------------------------------
 -- | Binds a list of compiled splices.  This function should not be exported.
@@ -535,8 +558,7 @@ bindSplices :: Splices (Splice n)  -- ^ splices to bind
             -> HeistState n        -- ^ source state
             -> HeistState n
 bindSplices ss hs =
-    hs { _compiledSpliceMap = H.union (runMapNoErrors ss)
-                                      (_compiledSpliceMap hs) }
+    hs { _compiledSpliceMap = applySpliceMap hs _compiledSpliceMap ss }
 
 
 ------------------------------------------------------------------------------
