@@ -76,7 +76,7 @@ type TPath = [ByteString]
 data DocumentFile = DocumentFile
     { dfDoc  :: X.Document
     , dfFile :: Maybe FilePath
-    } deriving ( Eq
+    } deriving ( Eq, Show
 #if MIN_VERSION_base(4,7,0)
                , Typeable
 #endif
@@ -204,6 +204,7 @@ data HeistState m = HeistState {
     -- correspond to a bound splice.  When not using a namespace, this flag is
     -- ignored.
     , _errorNotBound       :: Bool
+    , _numNamespacedTags   :: Int
 #if MIN_VERSION_base(4,7,0)
 } deriving (Typeable)
 #else
@@ -500,12 +501,14 @@ modifyHS f = HeistT $ \_ s -> return ((), f s)
 
 ------------------------------------------------------------------------------
 -- | Restores the HeistState.  This function is almost like putHS except it
--- preserves the current doctypes.  You should use this function instead of
--- @putHS@ to restore an old state.  This was needed because doctypes needs to
--- be in a "global scope" as opposed to the template call "local scope" of
--- state items such as recursionDepth, curContext, and spliceMap.
+-- preserves the current doctypes and splice errors.  You should use this
+-- function instead of @putHS@ to restore an old state.  This was needed
+-- because doctypes needs to be in a "global scope" as opposed to the template
+-- call "local scope" of state items such as recursionDepth, curContext, and
+-- spliceMap.
 restoreHS :: Monad m => HeistState n -> HeistT n m ()
-restoreHS old = modifyHS (\cur -> old { _doctypes = _doctypes cur })
+restoreHS old = modifyHS (\cur -> old { _doctypes = _doctypes cur
+                                      , _spliceErrors = _spliceErrors cur })
 {-# INLINE restoreHS #-}
 
 
@@ -532,6 +535,13 @@ modRecursionDepth f =
     modifyHS (\st -> st { _recursionDepth = f (_recursionDepth st) })
 
 
+------------------------------------------------------------------------------
+-- | Increments the namespaced tag count
+incNamespacedTags :: Monad m => HeistT n m ()
+incNamespacedTags =
+    modifyHS (\st -> st { _numNamespacedTags = _numNamespacedTags st + 1 })
+
+
 ------------------------------------------------------------------------------
 -- | AST to hold attribute parsing structure.  This is necessary because
 -- attoparsec doesn't support parsers running in another monad.
